Package: calibmsm
Title: Calibration Plots for the Transition Probabilities from
        Multistate Models
Version: 1.1.1
Authors@R: 
  c(
  person(given = "Alexander", family = "Pate", email = "alexander.pate@manchester.ac.uk", role = c("aut", "cre", "cph"),
         comment = c(ORCID = "0000-0002-0849-3458")),
  person(given = "Glen P", family = "Martin", email = "glen.martin@manchester.ac.uk", role = c("fnd", "rev"),
         comment = c(ORCID = "0000-0002-3410-9472"))
         )
Description: Assess the calibration of an existing (i.e. previously developed) multistate
  model through calibration plots. 
  Calibration is assessed using one of three methods. 1) Calibration methods for 
  binary logistic regression models applied at a fixed time point in conjunction 
  with inverse probability of censoring weights. 2) Calibration methods for 
  multinomial logistic regression models applied at a fixed time point in conjunction 
  with inverse probability of censoring weights. 3) Pseudo-values estimated using 
  the Aalen-Johansen estimator of observed risk. All methods are applied in conjunction
  with landmarking when required. These calibration plots evaluate the calibration 
  (in a validation cohort of interest) of the transition probabilities estimated from an 
  existing multistate model. While package development has focused on multistate 
  models, calibration plots can be produced for any model which utilises information 
  post baseline to update predictions (e.g. dynamic models); competing risks models; 
  or standard single outcome survival models, where predictions can be made at 
  any landmark time. Please see Pate et al. (2024) <doi:10.1002/sim.10094>
  and Pate et al. (2024) <https://alexpate30.github.io/calibmsm/articles/Overview.html>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.1
Imports: boot, dplyr, ggplot2, ggpubr, ggExtra, gridExtra, Hmisc,
        mstate, rms, stats, survival, tidyr, VGAM
Depends: R (>= 2.10)
LazyData: true
Suggests: covr, knitr, rmarkdown, R.rsp, testthat (>= 3.0.0)
VignetteBuilder: knitr, R.rsp
URL: https://alexpate30.github.io/calibmsm/
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-06-14 09:11:19 UTC; mbrxsap3
Author: Alexander Pate [aut, cre, cph]
    (<https://orcid.org/0000-0002-0849-3458>),
  Glen P Martin [fnd, rev] (<https://orcid.org/0000-0002-3410-9472>)
Maintainer: Alexander Pate <alexander.pate@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2024-06-14 09:40:02 UTC
Built: R 4.4.1; ; 2024-06-15 15:14:23 UTC; unix
