---
title: "A catalogue of links and descriptions for the vignettes/articles"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A catalogue of links and descriptions for the vignettes/articles}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

All vignettes can be found as articles on the website [https://alexpate30.github.io/calibmsm/index.html](https://alexpate30.github.io/calibmsm/index.html).

The [Overview](https://alexpate30.github.io/calibmsm/articles/overview_vignette.pdf) vignette is a guide on how to use the package by running through a clinical example. This should be used as the starting point for understanding how to use **calibmsm**. This vignette will be submitted for publication when it is ready and a reference will be added here.

The [BLR-IPCW-manual-bootstrap](https://alexpate30.github.io/calibmsm/articles/BLR-IPCW-manual-bootstrap.html) vignette showcases how to estimate a confidence interval manually using bootstrapping, and how to manually specify a function for estimating the inverse probability of censoring weights.

The [Calibration-curves-estimated-with-loess-smoothers](https://alexpate30.github.io/calibmsm/articles/Calibration-curves-estimated-with-loess-smoothers.html) vignette repeats the analyses from the [Overview](https://alexpate30.github.io/calibmsm/articles/overview_vignette.pdf) vignette using loess smoothers to estimate the calibration curves instead of restricted cubic splines.

The [Evaluation-of-estimation-of-IPCWs](https://alexpate30.github.io/calibmsm/articles/Evaluation-of-estimation-of-IPCWs.html) vignette explores whether the assumptions of the BLR-IPCW and psuedo-value methods are met in the illustrative example in the [Overview](https://alexpate30.github.io/calibmsm/articles/overview_vignette.pdf) vignette. In particular, the validity of the inverse probability of censoring weights when estimated using the internal procedures in **calibmsm** is evaluated.

The [Comparison-in-competing-risks-setting](https://alexpate30.github.io/calibmsm/articles/Comparison-in-competing-risks-setting.html) vignette uses **calibmsm** to assess the calibration of a standard competing risk model and compares results with the more commonly used graphical calibration curves.



